package ru.kristall.alfresco.module.bg.constraints;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.alfresco.repo.dictionary.constraint.ListOfValuesConstraint;
import org.alfresco.repo.security.authentication.AuthenticationUtil;
import org.alfresco.service.cmr.dictionary.ConstraintException;
import org.alfresco.service.cmr.repository.datatype.DefaultTypeConverter;
import org.alfresco.service.cmr.repository.datatype.TypeConversionException;
//import org.alfresco.repo.jscript.ScriptLogger;
import org.apache.log4j.Logger;

public class KriListOfValuesConstraint extends ListOfValuesConstraint {

   private static final String ERR_NO_VALUES = "d_dictionary.constraint.list_of_values.no_values";
   private static final String ERR_NON_STRING = "d_dictionary.constraint.string_length.non_string";
   private static final String ERR_INVALID_VALUE = "d_dictionary.constraint.list_of_values.invalid_value";

   private static KriListOfValuesService kriListOfValuesService;

   private Logger logger = Logger.getLogger(KriListOfValuesConstraint.class);

   public void setFdListOfValuesService( KriListOfValuesService kriListOfValuesService) {
	 if (logger.isDebugEnabled()) { logger.debug("setFdListOfValuesService is called"); }
     KriListOfValuesConstraint.kriListOfValuesService = kriListOfValuesService;
   }

   private List<String> allowedValues;
   private List<String> allowedValuesUpper;

   @Override
   public String toString() {
      StringBuilder sb = new StringBuilder(80);
      sb.append("FDListOfValues").append("[allowedValues=").append(
            getAllowedValues()).append(", caseSensitive=").append(
            isCaseSensitive()).append("]");
      return sb.toString();
   }

   /**
    * Get the allowed values. Note that these are <tt>String</tt> instances,
    * but may represent non-<tt>String</tt> values. It is up to the caller to
    * distinguish.
    *
    * @return Returns the values allowed
    */
   @Override
   public List<String> getAllowedValues() {
      if(logger.isDebugEnabled()){ logger.debug("getAllowedValues is called");  }

      String runAsUser = AuthenticationUtil.getRunAsUser();
      if ((runAsUser != null)
            && (!runAsUser.equals(AuthenticationUtil.getSystemUserName()))
            && (kriListOfValuesService != null)) {
         List<String> allowedForUser = kriListOfValuesService.getAllowedValues(getShortName()); // get allowed values for current user
         if(logger.isDebugEnabled()){ logger.debug("allowedForUser:"+allowedForUser.toString());  }            
         return allowedForUser;
      } else {
         if(logger.isDebugEnabled()){ logger.debug("allowedValues:"+this.allowedValues.toString());  }
         return this.allowedValues;
      }
   }

   private List<String> getAllowedValuesUpper() {
      String runAsUser = AuthenticationUtil.getRunAsUser();
      if ((runAsUser != null)
            && (!runAsUser.equals(AuthenticationUtil.getSystemUserName()))
            && (kriListOfValuesService != null)) {
         List<String> allowedForUser = kriListOfValuesService.getAllowedValues(getType()); // get allowed values for current user
         List<String> filteredList = new ArrayList<String>(allowedForUser.size());
         for (String allowed : allowedForUser) {
            if (this.allowedValuesUpper.contains(allowed.toUpperCase())) { filteredList.add(allowed);   }
         }

         return filteredList;
      } else {
         return this.allowedValuesUpper;
      }
   }

   /**
    * Set the values that are allowed by the constraint.
    *
    * @param values
    *            a list of allowed values
    */
   @SuppressWarnings("unchecked")
   @Override
   public void setAllowedValues(List allowedValues) {
      if (allowedValues == null) { allowedValues = new ArrayList<String>(0);  }
      int valueCount = allowedValues.size();
      this.allowedValues = Collections.unmodifiableList(allowedValues);
      // make the upper case versions
      this.allowedValuesUpper = new ArrayList<String>(valueCount);
      for (String allowedValue : this.allowedValues) {
         allowedValuesUpper.add(allowedValue.toUpperCase());
      }
   }

   @Override
   public void initialize() {
      checkPropertyNotNull("allowedValues", allowedValues);
   }

   @Override
   public Map<String, Object> getParameters() {
      Map<String, Object> params = new HashMap<String, Object>(2);
      params.put("caseSensitive", isCaseSensitive());
      params.put("allowedValues", getAllowedValues());
      return params;
   }

   @Override
   protected void evaluateSingleValue(Object value) {
      // convert the value to a String
      if(logger.isDebugEnabled()){ logger.debug("evaluateSingleValue is called"); }
      if(true) return;
      //РќРµ РїСЂРѕРІРµСЂСЏРµРј РЅР° СЃРѕРѕС‚РІРµС‚СЃС‚РІРёРµ С‚РёРїРѕРІ
      String valueStr = null;
      try {
         valueStr = DefaultTypeConverter.INSTANCE.convert(String.class, value);
      } catch (TypeConversionException e) {
         throw new ConstraintException(ERR_NON_STRING, value);
      }
      // check that the value is in the set of allowed values
      if (isCaseSensitive()) {
         List<String> allowedValues =getAllowedValues();
         if (!allowedValues.contains(valueStr)) {
            boolean found=false;            
            for(String allowValue : allowedValues){
               if(allowValue.indexOf(valueStr+"|")>=0){
                  found=true;
                  break;
               }               
            }
            if(!found)throw new ConstraintException(ERR_INVALID_VALUE, value);
         }
      } else {
         List<String> allowedValuesUpper =getAllowedValuesUpper();
         if (!allowedValuesUpper.contains(valueStr.toUpperCase())) {
            boolean found=false;            
            for(String allowValue : allowedValuesUpper){
               if(allowValue.indexOf(valueStr.toUpperCase()+"|")>=0){
                  found=true;
                  break;
               }               
            }
            if(!found)throw new ConstraintException(ERR_INVALID_VALUE, value);
         }
      }
   }
}